@@ -95,6 +95,16 @@ public:
   T& fromNSec(int64_t t);
   bool isZero() const;
   boost::posix_time::time_duration toBoost() const;
+  static const T MIN; //!< Minimum representable duration (negative)
+  static const T MAX; //!< Maximum representable duration
+  static const T ZERO; //!< Zero duration
+  static const T DAY; //!< One day duration
+  static const T HOUR; //!< One hour duration
+  static const T MINUTE; //!< One minute duration
+  static const T SECOND; //!< One second duration
+  static const T MILLISECOND; //!< One millisecond duration
+  static const T MICROSECOND; //!< One microsecond duration
+  static const T NANOSECOND; //!< One nanosecond duration
 };
 
 class Rate;
@@ -126,6 +136,16 @@ public:
 
 extern ROSTIME_DECL const Duration DURATION_MAX;
 extern ROSTIME_DECL const Duration DURATION_MIN;
+template<> const Duration DurationBase<Duration>::MAX;
+template<> const Duration DurationBase<Duration>::MIN;
+template<> const Duration DurationBase<Duration>::ZERO;
+template<> const Duration DurationBase<Duration>::DAY;
+template<> const Duration DurationBase<Duration>::HOUR;
+template<> const Duration DurationBase<Duration>::MINUTE;
+template<> const Duration DurationBase<Duration>::SECOND;
+template<> const Duration DurationBase<Duration>::MILLISECOND;
+template<> const Duration DurationBase<Duration>::MICROSECOND;
+template<> const Duration DurationBase<Duration>::NANOSECOND;
 
 /**
  * \brief Duration representation for use with the WallTime class.
@@ -152,6 +172,17 @@ public:
   bool sleep() const;
 };
 
+template<> const WallDuration DurationBase<WallDuration>::MAX;
+template<> const WallDuration DurationBase<WallDuration>::MIN;
+template<> const WallDuration DurationBase<WallDuration>::ZERO;
+template<> const WallDuration DurationBase<WallDuration>::DAY;
+template<> const WallDuration DurationBase<WallDuration>::HOUR;
+template<> const WallDuration DurationBase<WallDuration>::MINUTE;
+template<> const WallDuration DurationBase<WallDuration>::SECOND;
+template<> const WallDuration DurationBase<WallDuration>::MILLISECOND;
+template<> const WallDuration DurationBase<WallDuration>::MICROSECOND;
+template<> const WallDuration DurationBase<WallDuration>::NANOSECOND;
+
 ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const Duration &rhs);
 ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const WallDuration &rhs);
 
