@@ -160,6 +160,10 @@ namespace ros
     inline bool is_zero() const { return isZero(); }
     boost::posix_time::ptime toBoost() const;
 
+    static const T MIN; //!< Minimum representable time
+    static const T MAX; //!< Maximum representable time
+    static const T ZERO; //!< Zero (invalid) time
+    static const T UNINITIALIZED; //!< Uninitialized time
   };
 
   /**
@@ -217,6 +221,10 @@ namespace ros
 
   extern ROSTIME_DECL const Time TIME_MAX;
   extern ROSTIME_DECL const Time TIME_MIN;
+  template<> const Time TimeBase<Time, Duration>::MAX;
+  template<> const Time TimeBase<Time, Duration>::MIN;
+  template<> const Time TimeBase<Time, Duration>::ZERO;
+  template<> const Time TimeBase<Time, Duration>::UNINITIALIZED;
 
   /**
    * \brief Time representation.  Always wall-clock time.
@@ -249,7 +257,12 @@ namespace ros
 
     static bool isSystemTime() { return true; }
   };
-
+  
+  template<> const WallTime TimeBase<WallTime, WallDuration>::MAX;
+  template<> const WallTime TimeBase<WallTime, WallDuration>::MIN;
+  template<> const WallTime TimeBase<WallTime, WallDuration>::ZERO;
+  template<> const WallTime TimeBase<WallTime, WallDuration>::UNINITIALIZED;
+  
   /**
    * \brief Time representation.  Always steady-clock time.
    *
@@ -284,6 +297,11 @@ namespace ros
       static bool isSystemTime() { return true; }
   };
 
+  template<> const SteadyTime TimeBase<SteadyTime, WallDuration>::MAX;
+  template<> const SteadyTime TimeBase<SteadyTime, WallDuration>::MIN;
+  template<> const SteadyTime TimeBase<SteadyTime, WallDuration>::ZERO;
+  template<> const SteadyTime TimeBase<SteadyTime, WallDuration>::UNINITIALIZED;
+
   ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const Time &rhs);
   ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const WallTime &rhs);
   ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const SteadyTime &rhs);
